// SPDX-License-Identifier: GPL-3.0
//
// Copyright (C) 2019, Donald Dugger
//

import clock from "clock";
import { HeartRateSensor } from "heart-rate";
import { today } from 'user-activity';
import { charger, battery } from "power";
import { logger, screens, sh_screen } from "../common/utils";
import * as utils from "../common/utils";
import { tz_data } from "../common/tz";
import * as names from "../common/names";
import * as C from "../common/constants";
import { P } from "../common/prefs";
import * as cfg0 from "../common/config0";
import * as timer from "../common/timer";

let screen;
let scr;
let moon_day = -1;

var bpm_showing = true;
let hrm;
const days = ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"]

export function init(doc)
{

  let bar;
  screen = doc.getElementById("analog_face");
  scr = utils.scr_init(screen, null);
  utils.image_btn(screen, "lr_btn", "icons/list_btn.png", (btn) => {
    if (timer.running)
      timer.set_lap();
    else {
      //cfg0.set_time();
      cfg0.sh_screen();
    }
  });

  for (let i = 0; i < 12; i++) {
    bar = screen.getElementById("tick-" + i);
    bar.groupTransform.rotate.angle = (i * 30);
  }

  let bpm = screen.getElementById("bpm");
  bpm.addEventListener("click", (evt) => {
    bpm_showing = !bpm_showing;
    screen.getElementById("bpm_img").image = (bpm_showing ? "icons/bpm.png" :
                                                            "icons/steps.png");
    screen.getElementById("bpm").text = "??";
  })

  let tz_time2 = screen.getElementById("tz_time2");
  tz_time2.addEventListener("click", (evt) => {
    timer.sh_screen();
  })

  tz_data[C.TZ_CUST1_IDX].off = P.custom_tz1;
  tz_data[C.TZ_CUST2_IDX].off = P.custom_tz2;

  hrm = new HeartRateSensor();
  hrm.onreading = function() {
    if (bpm_showing)
      screen.getElementById("bpm").text = (hrm.heartRate || "--");
    hrm.stop();
  }

  set_rotation();
  set_moon("icons/list_btn.png");

  return 0;
}

export function set_rotation()
{

  let min3 = screen.getElementById("min-3");
  let min9 = screen.getElementById("min-9");
  if (P.rotation < 0) {
    min3.text = "9";
    min9.text = "3";
  } else {
    min3.text = "3";
    min9.text = "9";
  }
}

function moon_image(local)
{

  if (local == null)
    local = new Date();
  moon_day = local.getDate();
  return utils.moon_phase(local);
}

export function set_moon(image)
{

  if (image == null) {
    image = moon_image(null);
    image = P.moon ? image : "icons/list_btn.png";
  }
  let btn = screen.getElementById("lr_btn");
  let lr_icon = btn.getElementById("b_image");
  lr_icon.image = image;
}

export function updateMoon(local)
{

  if (P.moon && (local.getDate() != moon_day))
    set_moon(moon_image(local));
}

// Returns an angle (0-360) for the current hour in the day, including minutes
function hoursToAngle(hours, minutes) {
  let hourAngle = (360 / 12) * hours;
  let minAngle = (360 / 12 / 60) * minutes;
  return (hourAngle + minAngle) * P.rotation;
}

// Returns an angle (0-360) for minutes
function minutesToAngle(minutes) {
  return ((360 / 60) * minutes) * P.rotation;
}

// Returns an angle (0-360) for seconds
function secondsToAngle(seconds) {
  return ((360 / 60) * seconds) * P.rotation;
}

function updateBattery() {
  
  let lvl = Math.floor(battery.chargeLevel);
  let batValue = screen.getElementById("battery");
  batValue.text = lvl;
  if (lvl > 50)
    batValue.style.fill = "lime";
  else if (lvl > 25)
    batValue.style.fill = "yellow";
  else
    batValue.style.fill = "red";
}

function get_time2(tz2_idx, utc_time, tm2)
{
  var tz, time;
  
  if (tz2_idx == C.TZ_TIMER_IDX) {
    return timer.state();
  } else if (tz2_idx >= 0) {
    tz = utils.find_tz(tz2_idx, utc_time);
    let hr = tm2.getHours();
    let ampm = utils.ampm_mark(hr, P, " A", " P");
    if (!P.hr24 && (hr > 12))
      hr -= 12;
    let mn = tm2.getMinutes();
    time = utils.numPad(hr, " ") + ":" + utils.numPad(mn, "0");
    if (!P.hr24)
      time += ampm;
  } else {
    tz = "";
    time = "";
  }
  return { tz, time };
}

// Rotate the hands every tick
export function updateFace()
{

  let local = new Date();
  let tm1 = local;
  let tm2;
  let tz1_idx = names.get_info(P.tz1).tz;
  let tz2_idx = names.get_info(P.tz2).tz;
  if (tz1_idx > 0 || tz2_idx >= 0) {
    let off = utils.utc_off(tm1.toString());
    let utc_time = utils.utc(tm1, -off);
    if (tz1_idx > 0)
      tm1 = utils.utc(utc_time, utils.tz_off(tz1_idx, utc_time));
    if (tz2_idx > 0)
      tm2 = utils.utc(utc_time, utils.tz_off(tz2_idx, utc_time));
    else if (tz2_idx == 0)
      tm2 = local;
  }
  let hours = tm1.getHours();
  let mins = tm1.getMinutes();
  let secs = tm1.getSeconds();

  screen.getElementById("pm").text = utils.ampm_mark(hours, P, "AM", "PM");

  let hourHand = screen.getElementById("hours");
  hourHand.groupTransform.rotate.angle = hoursToAngle(hours%12, mins);

  let minHand = screen.getElementById("mins");
  minHand.groupTransform.rotate.angle = minutesToAngle(mins);

  let secHand = screen.getElementById("secs");
  secHand.groupTransform.rotate.angle = secondsToAngle(secs);

  screen.getElementById("tz1").text = ((tz1_idx == 0) ? "" : utils.find_tz(tz1_idx, utc_time));
  
  let { tz, time } = get_time2(tz2_idx, utc_time, tm2);
  screen.getElementById("tz2").text = tz;
  screen.getElementById("tz_time2").text = time;

  screen.getElementById("date").text = tm1.getDate();
  screen.getElementById("week").text = days[tm1.getDay()];

  updateMoon(local);
  
  updateBattery();
  
  if (bpm_showing)
    hrm.start();
  else
    screen.getElementById("bpm").text = utils.comma(today.adjusted.steps);
}

export function updateClock()
{

  if (!timer.running)
    updateFace();
}
